==> spring-boot-ext-1.2.195-sources.jar: cn/com/duibaboot/ext/autoconfigure/datasource/DuibaDataSourceProperties.java <==
package cn.com.duibaboot.ext.autoconfigure.datasource;

import cn.com.duibaboot.ext.autoconfigure.perftest.PerfTestRoutingDataSource;
import org.apache.commons.dbcp2.BasicDataSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DatabaseDriver;
import org.springframework.util.StringUtils;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@ConfigurationProperties("duiba.datasource")
public class DuibaDataSourceProperties extends HashMap<String, DuibaDataSourceProperties.DataSourcePropertiesInner> {

    public static class DataSourcePropertiesInner{

        private static final Logger logger = LoggerFactory.getLogger(DataSourcePropertiesInner.class);
        private static final Class<?> DBCP2_CLASS;

        static{
            Class<?> dbcp2Class = null;
            try {
                dbcp2Class = Class.forName("org.apache.commons.dbcp2.BasicDataSource");
            } catch (ClassNotFoundException e) {
                //Ignore
            }

            DBCP2_CLASS = dbcp2Class;
        }

        /**
         * JDBC url of the database.
         */
        private String url;

        /**
         * Login user of the database.
         */
        private String username;

        /**
         * Login password of the database.
         */
        private String password;

        /**
         * 连接池最大数量
         */
        private Integer maxTotal = 8;

        /**
         * 连接池初始值
         */
        private Integer initialSize = 3;

        /**
         * 连接池最大空闲
         */
        private Integer maxIdle = 8;

        /**
         * 连接池最小空闲
         */
        private Integer minIdle = 0;
        /**
         * 等待连接池的最大时间（ms）
         */
        private Integer maxWaitMillis = 3000;

        public String getUrl() {
            return url;
        }

        public void setUrl(String url) {
            this.url = url;
        }

        public String getUsername() {
            return username;
        }

        public void setUsername(String username) {
            this.username = username;
        }

        public String getPassword() {
            return password;
        }

        public void setPassword(String password) {
            this.password = password;
        }

        public Integer getMaxTotal() {
            return maxTotal;
        }

        public void setMaxTotal(Integer maxTotal) {
            this.maxTotal = maxTotal;
        }

        public Integer getInitialSize() {
            return initialSize;
        }

        public void setInitialSize(Integer initialSize) {
            this.initialSize = initialSize;
        }

        public Integer getMaxIdle() {
            return maxIdle;
        }

        public void setMaxIdle(Integer maxIdle) {
            this.maxIdle = maxIdle;
        }

        public Integer getMinIdle() {
            return minIdle;
        }

        public void setMinIdle(Integer minIdle) {
            this.minIdle = minIdle;
        }

        public Integer getMaxWaitMillis() {
            return maxWaitMillis;
        }

        public void setMaxWaitMillis(Integer maxWaitMillis) {
            this.maxWaitMillis = maxWaitMillis;
        }

        public Class<? extends DataSource> getType() {
            try {
                Class.forName("org.apache.commons.dbcp2.BasicDataSource");
            } catch (ClassNotFoundException e) {
                throw new IllegalStateException("检测到你的项目中需要使用dataSource，目前强制使用dbcp2，请引入dbcp2 的jar依赖.");
            }
            return BasicDataSource.class;
        }


        /**
         * Determine the driver to use based on this configuration and the environment.
         * @return the driver to use
         * @since 1.4.0
         */
        public String determineDriverClassName() {
//            if (StringUtils.hasText(this.driverClassName)) {
//                Assert.state(driverClassIsLoadable(),
//                        "Cannot load driver class: " + this.driverClassName);
//                return this.driverClassName;
//            }
            String driverClassName = null;

            if (StringUtils.hasText(this.url)) {
                driverClassName = DatabaseDriver.fromJdbcUrl(this.url).getDriverClassName();
            }

            if (!StringUtils.hasText(driverClassName)) {
                throw new IllegalStateException("can't determine driverClassName of url:" + url);
            }
            return driverClassName;
        }

        /**
         * Initialize a {@link BeanDefinitionBuilder} with the state of this instance.
         * @return a {@link BeanDefinitionBuilder} initialized with the customizations defined on
         * this instance
         */
        public BeanDefinitionBuilder initializeDataSourceBeanDefinitionBuilder() {
            if(StringUtils.isEmpty(url) || StringUtils.isEmpty(username) || StringUtils.isEmpty(password)){
                return null;
            }
            Class dataSourceClass = getType();
            if(dataSourceClass.getName().equals("org.apache.commons.dbcp2.BasicDataSource")){
                return BeanDefinitionBuilder.genericBeanDefinition(dataSourceClass)
                        .addPropertyValue("driverClassName", determineDriverClassName())
                        .addPropertyValue("url", getUrl())
                        .addPropertyValue("username", getUsername())
                        .addPropertyValue("password", getPassword())
                        .addPropertyValue("initialSize", initialSize)
                        .addPropertyValue("maxTotal", maxTotal)
                        .addPropertyValue("minIdle", minIdle)
                        .addPropertyValue("maxIdle", maxIdle)
                        .addPropertyValue("maxWaitMillis", maxWaitMillis)
                        .setDestroyMethodName("close");
            }

            throw new IllegalStateException("unsupportted dataSource type:" + dataSourceClass.getName() + ", currently only dbcp2 is supported");
        }

        /**
         * 运行时修改数据源配置
         * @param dataSourceBeanName spring bean id
         * @param dataSource 需要运行时修改配置的数据源
         */
        public void bindProperties(String dataSourceBeanName, DataSource dataSource){
            List<DataSource> dataSourcesNeedChange = new ArrayList<>();
            if(dataSource instanceof PerfTestRoutingDataSource){
                PerfTestRoutingDataSource ds1 = (PerfTestRoutingDataSource)dataSource;
                dataSourcesNeedChange.add(ds1.getOriginalDataSource());
                if(ds1.getShadeDataSource() != null) {
                    dataSourcesNeedChange.add(ds1.getShadeDataSource());
                }
            }else{
                dataSourcesNeedChange.add(dataSource);
            }

            for(DataSource ds : dataSourcesNeedChange){
                if(DBCP2_CLASS != null && DBCP2_CLASS.isAssignableFrom(ds.getClass())){
                    BasicDataSource d = (BasicDataSource)ds;
                    d.setUrl(getUrl());
                    d.setUsername(getUsername());
                    d.setPassword(getPassword());
                    d.setInitialSize(initialSize);
                    d.setMaxTotal(maxTotal);
                    d.setMinIdle(minIdle);
                    d.setMaxIdle(maxIdle);
                    d.setMaxWaitMillis(maxWaitMillis);
                }else{
                    logger.warn("不能刷新数据源类型：{}, beanId:{}，当前只支持刷新dbcp2数据源", ds.getClass().getName(), dataSourceBeanName);
                }
            }
        }
    }

}

==> spring-boot-ext-1.2.218-sources.jar: cn/com/duibaboot/ext/autoconfigure/datasource/DuibaDataSourceProperties.java <==
package cn.com.duibaboot.ext.autoconfigure.datasource;

import cn.com.duibaboot.ext.autoconfigure.perftest.PerfTestRoutingDataSource;
import org.apache.commons.dbcp2.BasicDataSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DatabaseDriver;
import org.springframework.util.StringUtils;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@ConfigurationProperties("duiba.datasource")
public class DuibaDataSourceProperties extends HashMap<String, DuibaDataSourceProperties.DataSourcePropertiesInner> {

    public static class DataSourcePropertiesInner{

        private static final Logger logger = LoggerFactory.getLogger(DataSourcePropertiesInner.class);
        private static final Class<?> DBCP2_CLASS;

        static{
            Class<?> dbcp2Class = null;
            try {
                dbcp2Class = Class.forName("org.apache.commons.dbcp2.BasicDataSource");
            } catch (ClassNotFoundException e) {
                //Ignore
            }

            DBCP2_CLASS = dbcp2Class;
        }

        /**
         * JDBC url of the database.
         */
        private String url;

        /**
         * Login user of the database.
         */
        private String username;

        /**
         * Login password of the database.
         */
        private String password;

        /**
         * 连接池最大数量
         */
        private Integer maxTotal = 8;

        /**
         * 连接池初始值
         */
        private Integer initialSize = 3;

        /**
         * 连接池最大空闲
         */
        private Integer maxIdle = 8;

        /**
         * 连接池最小空闲
         */
        private Integer minIdle = 0;
        /**
         * 等待连接池的最大时间（ms）
         */
        private Integer maxWaitMillis = 3000;

        private List<String> connectionInitSqls = new ArrayList<>();

        public String getUrl() {
            return url;
        }

        public void setUrl(String url) {
            this.url = url;
        }

        public String getUsername() {
            return username;
        }

        public void setUsername(String username) {
            this.username = username;
        }

        public String getPassword() {
            return password;
        }

        public void setPassword(String password) {
            this.password = password;
        }

        public Integer getMaxTotal() {
            return maxTotal;
        }

        public void setMaxTotal(Integer maxTotal) {
            this.maxTotal = maxTotal;
        }

        public Integer getInitialSize() {
            return initialSize;
        }

        public void setInitialSize(Integer initialSize) {
            this.initialSize = initialSize;
        }

        public Integer getMaxIdle() {
            return maxIdle;
        }

        public void setMaxIdle(Integer maxIdle) {
            this.maxIdle = maxIdle;
        }

        public Integer getMinIdle() {
            return minIdle;
        }

        public void setMinIdle(Integer minIdle) {
            this.minIdle = minIdle;
        }

        public Integer getMaxWaitMillis() {
            return maxWaitMillis;
        }

        public void setMaxWaitMillis(Integer maxWaitMillis) {
            this.maxWaitMillis = maxWaitMillis;
        }

        public Class<? extends DataSource> getType() {
            try {
                Class.forName("org.apache.commons.dbcp2.BasicDataSource");
            } catch (ClassNotFoundException e) {
                throw new IllegalStateException("检测到你的项目中需要使用dataSource，目前强制使用dbcp2，请引入dbcp2 的jar依赖.");
            }
            return BasicDataSource.class;
        }


        /**
         * Determine the driver to use based on this configuration and the environment.
         * @return the driver to use
         * @since 1.4.0
         */
        public String determineDriverClassName() {
//            if (StringUtils.hasText(this.driverClassName)) {
//                Assert.state(driverClassIsLoadable(),
//                        "Cannot load driver class: " + this.driverClassName);
//                return this.driverClassName;
//            }
            String driverClassName = null;

            if (StringUtils.hasText(this.url)) {
                driverClassName = DatabaseDriver.fromJdbcUrl(this.url).getDriverClassName();
                if(driverClassName == null){
                    if(this.url.startsWith("jdbc:phoenix:")){//phoenix
                        driverClassName = "org.apache.phoenix.jdbc.PhoenixDriver";
                    }
                }
            }

            if (!StringUtils.hasText(driverClassName)) {
                throw new IllegalStateException("can't determine driverClassName of url:" + url);
            }
            return driverClassName;
        }

        /**
         * Initialize a {@link BeanDefinitionBuilder} with the state of this instance.
         * @return a {@link BeanDefinitionBuilder} initialized with the customizations defined on
         * this instance
         */
        public BeanDefinitionBuilder initializeDataSourceBeanDefinitionBuilder() {
            if(StringUtils.isEmpty(url)){
                return null;
            }
            Class dataSourceClass = getType();
            if(DBCP2_CLASS != null && DBCP2_CLASS.isAssignableFrom(dataSourceClass)){
                BeanDefinitionBuilder builder = BeanDefinitionBuilder.genericBeanDefinition(dataSourceClass)
                        .addPropertyValue("driverClassName", determineDriverClassName())
                        .addPropertyValue("url", getUrl())
                        .addPropertyValue("initialSize", initialSize)
                        .addPropertyValue("maxTotal", maxTotal)
                        .addPropertyValue("minIdle", minIdle)
                        .addPropertyValue("maxIdle", maxIdle)
                        .addPropertyValue("maxWaitMillis", maxWaitMillis)
                        .setDestroyMethodName("close");

                if(!org.apache.commons.lang3.StringUtils.isBlank(getUsername())) {
                    builder.addPropertyValue("username", getUsername());
                }
                if(!org.apache.commons.lang3.StringUtils.isBlank(getPassword())) {
                    builder.addPropertyValue("password", getPassword());
                }
                builder.addPropertyValue("connectionInitSqls", getConnectionInitSqls());

                return builder;
            }

            throw new IllegalStateException("unsupportted dataSource type:" + dataSourceClass.getName() + ", currently only dbcp2 is supported");
        }

        /**
         * 运行时修改数据源配置
         * @param dataSourceBeanName spring bean id
         * @param dataSource 需要运行时修改配置的数据源
         */
        public void bindProperties(String dataSourceBeanName, DataSource dataSource){
            List<DataSource> dataSourcesNeedChange = new ArrayList<>();
            if(dataSource instanceof PerfTestRoutingDataSource){
                PerfTestRoutingDataSource ds1 = (PerfTestRoutingDataSource)dataSource;
                dataSourcesNeedChange.add(ds1.getOriginalDataSource());
                if(ds1.getShadeDataSource() != null) {
                    dataSourcesNeedChange.add(ds1.getShadeDataSource());
                }
            }else{
                dataSourcesNeedChange.add(dataSource);
            }

            for(DataSource ds : dataSourcesNeedChange){
                if(DBCP2_CLASS != null && DBCP2_CLASS.isAssignableFrom(ds.getClass())){
                    BasicDataSource d = (BasicDataSource)ds;
                    d.setUrl(getUrl());
                    d.setUsername(getUsername());
                    d.setPassword(getPassword());
                    d.setInitialSize(initialSize);
                    d.setMaxTotal(maxTotal);
                    d.setMinIdle(minIdle);
                    d.setMaxIdle(maxIdle);
                    d.setMaxWaitMillis(maxWaitMillis);
                }else{
                    logger.warn("不能刷新数据源类型：{}, beanId:{}，当前只支持刷新dbcp2数据源", ds.getClass().getName(), dataSourceBeanName);
                }
            }
        }

        public List<String> getConnectionInitSqls() {
            return connectionInitSqls;
        }

        public void setConnectionInitSqls(List<String> connectionInitSqls) {
            this.connectionInitSqls = connectionInitSqls;
        }
    }

}
